@@ -14,6 +14,7 @@ from sqlalchemy.engine.strategies import MockEngineStrategy
 
 from .. import ddl
 from .. import util
+from ..util import sqla_compat
 from ..util.compat import callable
 from ..util.compat import EncodedIO
 
@@ -205,6 +206,7 @@ class MigrationContext(object):
                     "got %r" % connection,
                     stacklevel=3,
                 )
+
             dialect = connection.dialect
         elif url:
             url = sqla_url.make_url(url)
@@ -442,7 +444,7 @@ class MigrationContext(object):
             self.connection.execute(self._version.delete())
 
     def _has_version_table(self):
-        return self.connection.dialect.has_table(
+        return sqla_compat._connectable_has_table(
             self.connection, self.version_table, self.version_table_schema
         )
 
@@ -495,7 +497,9 @@ class MigrationContext(object):
         else:
             heads = self.get_current_heads()
 
-            if not self.as_sql and not heads:
+            dont_mutate = self.opts.get("dont_mutate", False)
+
+            if not self.as_sql and not heads and not dont_mutate:
                 self._ensure_version_table()
 
         head_maintainer = HeadMaintainer(self, heads)
@@ -676,7 +680,11 @@ class HeadMaintainer(object):
                 == literal_column("'%s'" % version)
             )
         )
-        if not self.context.as_sql and ret.rowcount != 1:
+        if (
+            not self.context.as_sql
+            and self.context.dialect.supports_sane_rowcount
+            and ret.rowcount != 1
+        ):
             raise util.CommandError(
                 "Online migration expected to match one "
                 "row when deleting '%s' in '%s'; "
@@ -697,7 +705,11 @@ class HeadMaintainer(object):
                 == literal_column("'%s'" % from_)
             )
         )
-        if not self.context.as_sql and ret.rowcount != 1:
+        if (
+            not self.context.as_sql
+            and self.context.dialect.supports_sane_rowcount
+            and ret.rowcount != 1
+        ):
             raise util.CommandError(
                 "Online migration expected to match one "
                 "row when updating '%s' to '%s' in '%s'; "
